#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class ForeignBornDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "foreignborn"
    UNIVERSE = "total population"
    RAW_TABLE_NAME = 'B05002'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "universe",
        "002": "native",
        "013": "foreign_born"
    })


@register
class CitizenDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "citizenstatus"
    UNIVERSE = "total population"
    RAW_TABLE_NAME = "B05001"
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "universe",
        "002": "us_citizen_born_us",
        "003": "us_citizen_born_puertorico_or_us_island",
        "004": "us_citizen_born_abroad_american_parents",
        "005": "us_citizen_by_naturalization",
        "006": "not_us_citizen"
    })

    def process(self, *args, **kwargs):
        df = super().process(*args, **kwargs)
        # Our custom race groups
        df['us_citizen_total'] = df[[
            'us_citizen_born_us',
            'us_citizen_born_puertorico_or_us_island',
            'us_citizen_born_abroad_american_parents',
            'us_citizen_by_naturalization'
        ]].sum(axis=1)
        return df



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class HouseholdIncomeDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "householdincome"
    UNIVERSE = "households"
    RAW_TABLE_NAME = 'B19001'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "10_and_under",
        '003': "10_15",
        '004': "15_20",
        '005': "20_25",
        '006': "25_30",
        '007': "30_35",
        '008': "35_40",
        '009': "40_45",
        '010': "45_50",
        '011': "50_60",
        '012': "60_75",
        '013': "75_100",
        '014': "100_125",
        '015': "125_150",
        '016': "150_200",
        '017': "200_and_up"
    })


@register
class HouseholdIncomeLatinoDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomelatino"
    RAW_TABLE_NAME = 'B19001I'


@register
class HouseholdIncomeWhiteDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomewhite"
    RAW_TABLE_NAME = 'B19001H'


@register
class HouseholdIncomeBlackDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomeblack"
    RAW_TABLE_NAME = 'B19001B'


@register
class HouseholdIncomeAsianDownloader(HouseholdIncomeDownloader):
    PROCESSED_TABLE_NAME = "householdincomeasian"
    RAW_TABLE_NAME = 'B19001D'



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class MedianMonthlyHousingCostsDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "medianmonthlyhousingcosts"
    UNIVERSE = "occupied housing units with monthly housing costs"
    RAW_TABLE_NAME = "B25105"
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "median"
    })


@register
class MedianHousingValueDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "medianhousingvalue"
    UNIVERSE = "owner-occupied housing units"
    RAW_TABLE_NAME = 'B25077'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "median"
    })


@register
class MedianGrossRentDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "mediangrossrent"
    UNIVERSE = "renter-occupied housing units paying cash rent"
    RAW_TABLE_NAME = 'B25064'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        "001": "median"
    })


@register
class TenureDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "tenure"
    UNIVERSE = "occupied housing units"
    RAW_TABLE_NAME = 'B25003'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': 'universe',
        '002': 'owner_occupied',
        '003': 'renter_occupied'
    })


@register
class TenureLatinoDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenurelatino"
    RAW_TABLE_NAME = 'B25003I'


@register
class TenureWhiteDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenurewhite"
    RAW_TABLE_NAME = 'B25003H'


@register
class TenureBlackDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenureblack"
    RAW_TABLE_NAME = 'B25003B'


@register
class TenureAsianDownloader(TenureDownloader):
    PROCESSED_TABLE_NAME = "tenureasian"
    RAW_TABLE_NAME = 'B25003D'



#! /usr/bin/env python
# -*- coding: utf-8 -*
# flake8: NOQA
import logging
from .tables import TABLE_LIST
logger = logging.getLogger(__name__)


def download_everything(*args, **kwargs):
    """
    Download all the data.
    """
    logger.debug("Downloading all datasets for all geographies")
    for klass in TABLE_LIST:
        obj = klass(*args, **kwargs)
        logger.debug(f"Downloading {klass.PROCESSED_TABLE_NAME} dataset")
        obj.download_everything()


__all__ = ("TABLE_LIST", "download_everything")
